@@ -2,6 +2,7 @@ import copy
 import json
 import os
 import shutil
+import warnings
 from argparse import Namespace
 from os import makedirs
 from os.path import basename, join, normpath
@@ -345,23 +346,52 @@ class Dataset:
 
     def get_segmentation_layer(self) -> SegmentationLayer:
         """
-        Returns the only segmentation layer.
+        Returns the only segmentation layer. Prefer `get_segmentation_layers`,
+        because multiple segmentation layers can exist.
 
         Fails with a IndexError if there are multiple segmentation layers or none.
         """
+
+        warnings.warn(
+            "[DEPRECATION] get_segmentation_layer() fails if no or more than one segmentation layer exists. Prefer get_segmentation_layers()."
+        )
         return cast(
             SegmentationLayer,
             self._get_layer_by_category(SEGMENTATION_CATEGORY),
         )
 
+    def get_segmentation_layers(self) -> List[SegmentationLayer]:
+        """
+        Returns all segmentation layers.
+        """
+        return [
+            cast(SegmentationLayer, layer)
+            for layer in self.layers.values()
+            if layer.category == SEGMENTATION_CATEGORY
+        ]
+
     def get_color_layer(self) -> Layer:
         """
-        Returns the only color layer.
+        Returns the only color layer. Prefer `get_color_layers`, because multiple
+        color layers can exist.
 
         Fails with a RuntimeError if there are multiple color layers or none.
         """
         return self._get_layer_by_category(COLOR_CATEGORY)
 
+    def get_color_layers(self) -> List[Layer]:
+        """
+        Returns all color layers.
+        """
+        warnings.warn(
+            "[DEPRECATION] get_color_layer() fails if no or more than one color layer exists. Prefer get_color_layers()."
+        )
+        return [
+            cast(Layer, layer)
+            for layer in self.layers.values()
+            if layer.category == COLOR_CATEGORY
+        ]
+
     def delete_layer(self, layer_name: str) -> None:
         """
         Deletes the layer from the `datasource-properties.json` and the data from disk.
