@@ -1,4 +1,3 @@
-import copy
 import logging
 import math
 import operator
@@ -27,6 +26,7 @@ from .downsampling_utils import (
     downsample_cube_job,
     parse_interpolation_mode,
 )
+from .layer_categories import COLOR_CATEGORY, SEGMENTATION_CATEGORY, LayerCategoryType
 from .properties import (
     LayerProperties,
     LayerViewConfiguration,
@@ -141,19 +141,6 @@ def _element_class_to_dtype_per_channel(
 class Layer:
     """
     A `Layer` consists of multiple `webknossos.dataset.mag_view.MagView`s, which store the same data in different magnifications.
-
-    ## Examples
-
-    ### Adding layer to dataset
-    ```python
-    from webknossos.dataset.dataset import Dataset
-
-    dataset = Dataset(<path_to_dataset>)
-    # Adds a new layer
-    layer = dataset.get_layer("color")
-    ```
-
-    ## Functions
     """
 
     def __init__(self, dataset: "Dataset", properties: LayerProperties) -> None:
@@ -218,10 +205,9 @@ class Layer:
         # The MagViews need to be updated
         for mag in self._mags.values():
             mag._path = _find_mag_path_on_disk(self.dataset.path, self.name, mag.name)
-            if mag._is_opened:
-                # Reopen handle to dataset on disk
-                mag.close()
-                mag.open()
+            # Deleting the dataset will close the file handle.
+            # The new dataset will be opened automatically when needed.
+            del mag._wkw_dataset
 
         self.dataset._export_as_json()
 
@@ -269,6 +255,10 @@ class Layer:
             )
         return self.mags[mag]
 
+    def get_best_mag(self) -> MagView:
+
+        return self.get_mag(min(self.mags.keys()))
+
     def add_mag(
         self,
         mag: Union[int, str, list, tuple, np.ndarray, Mag],
@@ -310,6 +300,29 @@ class Layer:
 
         return self._mags[mag]
 
+    def add_mag_for_existing_files(
+        self,
+        mag: Union[int, str, list, tuple, np.ndarray, Mag],
+    ) -> MagView:
+        """
+        Creates a new mag based on already existing files.
+
+        Raises an IndexError if the specified `mag` does not exists.
+        """
+        mag = Mag(mag)
+        assert (
+            mag not in self.mags
+        ), f"Cannot add mag {mag} as it already exists for layer {self}"
+        self._setup_mag(mag)
+        mag_view = self._mags[mag]
+        cube_length = mag_view.header.file_len * mag_view.header.block_len
+        self._properties.wkw_resolutions.append(
+            MagViewProperties(resolution=mag, cube_length=cube_length)
+        )
+        self.dataset._export_as_json()
+
+        return mag_view
+
     def get_or_add_mag(
         self,
         mag: Union[int, str, list, tuple, np.ndarray, Mag],
@@ -372,71 +385,93 @@ class Layer:
         rmtree(full_path)
 
     def _add_foreign_mag(
-        self, foreign_mag_path: Path, symlink: bool, make_relative: bool
+        self,
+        foreign_mag_view_or_path: Union[os.PathLike, str, MagView],
+        symlink: bool,
+        make_relative: bool,
+        extend_layer_bounding_box: bool = True,
     ) -> MagView:
-        mag_name = foreign_mag_path.name
-        mag = Mag(mag_name)
-        operation = "symlink" if symlink else "copy"
-        if mag in self.mags.keys():
-            raise IndexError(
-                f"Cannot {operation} {foreign_mag_path}. This dataset already has a mag called {mag_name}."
+        """
+        The foreign mag is (shallow) copied and the existing mag is added to the datasource-properties.json.
+        If extend_layer_bounding_box is true, the self.bounding_box will be extended
+        by the bounding box of the layer the foreign mag belongs to.
+        """
+
+        if isinstance(foreign_mag_view_or_path, MagView):
+            foreign_mag_view = foreign_mag_view_or_path
+        else:
+            # local import to prevent circular dependency
+            from .dataset import Dataset
+
+            foreign_mag_view_path = Path(foreign_mag_view_or_path)
+            foreign_mag_view = (
+                Dataset(foreign_mag_view_path.parent.parent)
+                .get_layer(foreign_mag_view_path.parent.name)
+                .get_mag(foreign_mag_view_path.name)
             )
 
+        self._assert_mag_does_not_exist_yet(foreign_mag_view.mag)
+
         foreign_normalized_mag_path = (
-            Path(os.path.relpath(foreign_mag_path, self.dataset.path))
+            Path(os.path.relpath(foreign_mag_view.path, self.path))
             if make_relative
-            else foreign_mag_path
+            else Path(os.path.abspath(foreign_mag_view.path))
         )
 
         if symlink:
             os.symlink(
                 foreign_normalized_mag_path,
-                join(self.dataset.path, self.name, mag_name),
+                join(self.dataset.path, self.name, str(foreign_mag_view.mag)),
             )
         else:
             shutil.copytree(
                 foreign_normalized_mag_path,
-                join(self.dataset.path, self.name, mag_name),
+                join(self.dataset.path, self.name, str(foreign_mag_view.mag)),
             )
 
-        # copy the properties of the layer into the properties of this dataset
-        from .dataset import Dataset  # local import to prevent circular dependency
-
-        original_layer = Dataset(foreign_mag_path.parent.parent).get_layer(
-            foreign_mag_path.parent.name
-        )
-        original_mag = original_layer.get_mag(foreign_mag_path.name)
-        mag_properties = copy.deepcopy(original_mag._properties)
-
-        self.bounding_box = self.bounding_box.extended_by(original_layer.bounding_box)
-        self._properties.wkw_resolutions += [mag_properties]
-        self._setup_mag(mag)
+        self.add_mag_for_existing_files(foreign_mag_view.mag)
+        if extend_layer_bounding_box:
+            self.bounding_box = self.bounding_box.extended_by(
+                foreign_mag_view.layer.bounding_box
+            )
         self.dataset._export_as_json()
-        return self.mags[mag]
+
+        return self._mags[foreign_mag_view.mag]
 
     def add_symlink_mag(
-        self, foreign_mag_path: Union[str, Path], make_relative: bool = False
+        self,
+        foreign_mag_view_or_path: Union[os.PathLike, str, MagView],
+        make_relative: bool = False,
+        extend_layer_bounding_box: bool = True,
     ) -> MagView:
         """
-        Creates a symlink to the data at `foreign_mag_path` which belongs to another dataset.
+        Creates a symlink to the data at `foreign_mag_view_or_path` which belongs to another dataset.
         The relevant information from the `datasource-properties.json` of the other dataset is copied to this dataset.
         Note: If the other dataset modifies its bounding box afterwards, the change does not affect this properties
         (or vice versa).
         If make_relative is True, the symlink is made relative to the current dataset path.
         """
-        foreign_mag_path = Path(os.path.abspath(foreign_mag_path))
         return self._add_foreign_mag(
-            foreign_mag_path, symlink=True, make_relative=make_relative
+            foreign_mag_view_or_path,
+            symlink=True,
+            make_relative=make_relative,
+            extend_layer_bounding_box=extend_layer_bounding_box,
         )
 
-    def add_copy_mag(self, foreign_mag_path: Union[str, Path]) -> MagView:
+    def add_copy_mag(
+        self,
+        foreign_mag_view_or_path: Union[os.PathLike, str, MagView],
+        extend_layer_bounding_box: bool = True,
+    ) -> MagView:
         """
-        Copies the data at `foreign_mag_path` which belongs to another dataset to the current dataset.
+        Copies the data at `foreign_mag_view_or_path` which belongs to another dataset to the current dataset.
         Additionally, the relevant information from the `datasource-properties.json` of the other dataset are copied too.
         """
-        foreign_mag_path = Path(os.path.abspath(foreign_mag_path))
         return self._add_foreign_mag(
-            foreign_mag_path, symlink=False, make_relative=False
+            foreign_mag_view_or_path,
+            symlink=False,
+            make_relative=False,
+            extend_layer_bounding_box=extend_layer_bounding_box,
         )
 
     def _create_dir_for_mag(
@@ -468,11 +503,9 @@ class Layer:
         self._properties.bounding_box = bbox
 
         for mag, mag_view in self.mags.items():
-            mag_view._size = (
-                self._properties.bounding_box.align_with_mag(mag, ceil=True)
-                .in_mag(mag)
-                .bottomright
-            )
+            bbox_in_mag = bbox.align_with_mag(mag, ceil=True).in_mag(mag)
+            mag_view._size = bbox_in_mag.bottomright
+            # The offset is not updated since a MagView always is in global coordinates.
         self.dataset._export_as_json()
 
     def downsample(
@@ -575,6 +608,7 @@ class Layer:
         compress: bool = True,
         buffer_edge_len: Optional[int] = None,
         args: Optional[Namespace] = None,
+        allow_overwrite: bool = False,
     ) -> None:
         """
         Performs a single downsampling step from `from_mag` to `target_mag`.
@@ -587,6 +621,7 @@ class Layer:
          - "bicubic"
 
         The `args` can contain information to distribute the computation.
+        If allow_overwrite is True, an existing Mag may be overwritten.
         """
         assert (
             from_mag in self.mags.keys()
@@ -597,7 +632,9 @@ class Layer:
         )
 
         assert from_mag <= target_mag
-        assert target_mag not in self.mags
+        assert (
+            allow_overwrite or target_mag not in self.mags
+        ), "The target mag already exists. Pass allow_overwrite=True if you want to overwrite it."
 
         prev_mag_view = self.mags[from_mag]
 
@@ -605,10 +642,13 @@ class Layer:
             t // s for (t, s) in zip(target_mag.to_list(), from_mag.to_list())
         ]
 
-        # initialize the new mag
-        target_mag_view = self._initialize_mag_from_other_mag(
-            target_mag, prev_mag_view, compress
-        )
+        if target_mag in self.mags.keys() and allow_overwrite:
+            target_mag_view = self.get_mag(target_mag)
+        else:
+            # initialize the new mag
+            target_mag_view = self._initialize_mag_from_other_mag(
+                target_mag, prev_mag_view, compress
+            )
 
         bb_mag1 = self.bounding_box
 
@@ -662,6 +702,34 @@ class Layer:
 
         logging.info("Mag {0} successfully cubed".format(target_mag))
 
+    def redownsample(
+        self,
+        interpolation_mode: str = "default",
+        compress: bool = True,
+        buffer_edge_len: Optional[int] = None,
+        args: Optional[Namespace] = None,
+    ) -> None:
+        """
+        Use this method to recompute downsampled magnifications after mutating data in the
+        base magnification.
+        """
+
+        mags = sorted(self.mags.keys(), key=lambda m: m.to_list())
+        if len(mags) <= 1:
+            # No downsampled magnifications exist. Thus, there's nothing to do.
+            return
+        from_mag = mags[0]
+        target_mags = mags[1:]
+        self.downsample_mag_list(
+            from_mag,
+            target_mags,
+            interpolation_mode,
+            compress,
+            buffer_edge_len,
+            args,
+            allow_overwrite=True,
+        )
+
     def downsample_mag_list(
         self,
         from_mag: Mag,
@@ -670,6 +738,7 @@ class Layer:
         compress: bool = True,
         buffer_edge_len: Optional[int] = None,
         args: Optional[Namespace] = None,
+        allow_overwrite: bool = False,
     ) -> None:
         """
         Downsamples the data starting at `from_mag` to each magnification in `target_mags` iteratively.
@@ -700,6 +769,7 @@ class Layer:
                 compress=compress,
                 buffer_edge_len=buffer_edge_len,
                 args=args,
+                allow_overwrite=allow_overwrite,
             )
             source_mag = target_mag
 
@@ -834,8 +904,8 @@ class Layer:
         )
 
     @property
-    def category(self) -> str:
-        return LayerCategories.COLOR_TYPE
+    def category(self) -> LayerCategoryType:
+        return COLOR_CATEGORY
 
     @property
     def dtype_per_layer(self) -> str:
@@ -858,22 +928,5 @@ class SegmentationLayer(Layer):
         self.dataset._export_as_json()
 
     @property
-    def category(self) -> str:
-        return LayerCategories.SEGMENTATION_TYPE
-
-
-class LayerCategories:
-    """
-    There are two different types of layers.
-    This class can be used to specify the type of a layer during creation:
-    ```python
-    from webknossos.dataset.dataset import Dataset
-
-    dataset = Dataset(<path_to_dataset>)
-    # Adds a new layer
-    layer = dataset.add_layer("color", LayerCategories.COLOR_TYPE)
-    ```
-    """
-
-    COLOR_TYPE = "color"
-    SEGMENTATION_TYPE = "segmentation"
+    def category(self) -> LayerCategoryType:
+        return SEGMENTATION_CATEGORY
